      program drivesub
c
c --- Driver for subroutine SUBDOMN
c                D. Strimaitis
c
c --- SUBDOMN    Version: 1.2      Level: 080620                  MAIN
c
c     Copyright (c) 2008 by Exponent, Inc.
c
      character*132 halt132
      character*132 inpnam, outnam, lstnam, inpfile
      character*16 clatlo,clonlo,clathi,clonhi
      character*6 ctype

      character*4 utmhem
      character*8 datum, pmap
      character*16 clat1,clat2,clat0,clon0

      write(*,*)'Enter the control file:  '
      read(*,*) inpfile

      open(100,file=inpfile)

c --- Files
      read(100,*) inpnam
      read(100,*) outnam
      read(100,*) lstnam

c --- File type
      read(100,*) ctype

c --- Range type
      read(100,*) mrange

c --- Extraction range
      if(mrange.EQ.1) then
         read(100,*) clatlo
         read(100,*) clathi
         read(100,*) clonlo
         read(100,*) clonhi
      elseif(mrange.EQ.2) then
         read(100,*) ylo
         read(100,*) yhi
         read(100,*) xlo
         read(100,*) xhi
      else
         write(*,*)'Problem with sub-domain range type: ',mrange
         write(*,*)'Expected 1 or 2'
         stop
      endif

c --- Ouput projection
      read(100,*) pmap
      read(100,*) datum
      read(100,*) utmhem
      read(100,*) iutmzn
      read(100,*) feast
      read(100,*) fnorth
      read(100,*) clat1
      read(100,*) clat2
      read(100,*) clat0
      read(100,*) clon0

      call subdomn(ctype,inpnam,outnam,lstnam,mrange,
     &             clatlo,clathi,clonlo,clonhi,
     &             ylo,yhi,xlo,xhi,
     &             iutmzn,feast,fnorth,
     &             pmap,datum,utmhem,clat0,clon0,clat1,clat2,
     &             halt132)

      if(halt132.NE.'OK') then
         write(*,*)'Problem with sub-domain run.  Message returned:'
         write(*,*) halt132
      endif

      close(100)

      stop
      end

c----------------------------------------------------------------------
      subroutine subdomn(ctype,inpnam,outnam,lstnam,mrange,
     &                   clatlo,clathi,clonlo,clonhi,
     &                   ylo,yhi,xlo,xhi,
     &                   iutmzni,feasti,fnorthi,pmapi,datumi,utmhemi,
     &                   clat0i,clon0i,clat1i,clat2i,
     &                   halt132)
c----------------------------------------------------------------------
c
c --- SUBDOMN    Version: 1.2      Level: 080620                  MAIN
c                D. Strimaitis
c
c --- PURPOSE:  Extract from a master data file stations that lie in
c               a subdomain.  May operate on files of type SURF.DAT,
c               PRECIP.DAT, or OZONE.DAT.
c               Output SURF.DAT/PRECIP.DAT format is 2.0 (map=NONE)
c               Corresponding CALMET.INP station data written to LIST
c               Output OZONE.DAT format is 2.0 (user map & datum)
c
c --- Note:     Place individual CALUTILS routines explicitly into
c               code to facilitate conversion to a DLL.
c               (ALLCAP, DEBLNK, INCR, XTRACTLL, BASRUTC)
c
c
c --- Version 1.1, Level 060210 to Version 1.2, Level 080620
c
c    (1) Update COORDLIB from Version 1.95, Level 050126
c                          to Version 1.99, Level 070921
c    (2) Remove subroutine GLOBE1 as it is now part of COORDLIB

c --- Version 1.0, Level 051128 to Version 1.1, Level 060210
c
c    (1) Add capability to accept extraction domain in either
c        Lat/Lon or X/Y (CALMET map and datum).  5 new arguments are
c        added for the range, and for the switch that controls which
c        format is used:
c
c        MRANGE   -  integer  - Flag for type of range coordinates
c                               1:  Lat/Lon (deg) as c*16 strings
c                               2:  X/Y (km) in CALMET map projection
c                                   and datum
c
c --- SUBDOMN calls:  PASSMAP, XTRACTLL, RDWRHD, RDWRDAT
c----------------------------------------------------------------------
c --- Parameters and commons
      include 'params.sdm'
      include 'control.sdm'

      character*132 halt132
      character*132 inpnam, outnam, lstnam
      character*6 ctype

      character*4 utmhemi
      character*8 datumi, pmapi
      character*16 clat1i,clat2i,clat0i,clon0i
      character*16 clatlo,clonlo,clathi,clonhi

      integer map(maxss)

c --- Set version and level
      ver='1.2'
      level='080620'

c --- Set return string
      halt132='OK'

c --- Process file type
      itype=0
      if(ctype.EQ.'SURF  ') itype=1
      if(ctype.EQ.'PRECIP') itype=2
      if(ctype.EQ.'OZONE ') itype=3
c --- Check the file type
      if(itype.EQ.0) then
         halt132='Unknown type of file requested: '//ctype
         return
      endif

c --- Send the output mapping information to /CONTROL/
      call PASSMAP(iutmzni,feasti,fnorthi,pmapi,datumi,utmhemi,
     &             clat0i,clon0i,clat1i,clat2i,
     &             halt132)
      if(halt132.NE.'OK') return

c --- Master File
      open(iinp,file=inpnam,status='old')

c --- NEW  File
      open(iout,file=outnam,status='unknown')

c --- List file
      open(ilst,file=lstnam,status='unknown')

c --- Move range coordinates into generic easting/northing variables
      if(mrange.EQ.1) then
c ---    LAT/LON (deg)
c ---    Translate character lat/lon to real NLat/ELon
         call XTRACTLL(ilst,'LAT ',clatlo,xlatlo)
         call XTRACTLL(ilst,'LAT ',clathi,xlathi)
         call XTRACTLL(ilst,'LON ',clonlo,xlonlo)
         call XTRACTLL(ilst,'LON ',clonhi,xlonhi)
c ---    Check N. Latitude order
         if(xlathi.LT.xlatlo) then
            xlat=xlathi
            xlathi=xlatlo
            xlatlo=xlat
         endif
c ---    Check E. Longitude order
         if(xlonhi.LT.xlonlo) then
            xlon=xlonhi
            xlonhi=xlonlo
            xlonlo=xlon
         endif
c ---    Assign to generics
         xe1=xlonlo
         xe2=xlonhi
         yn1=xlatlo
         yn2=xlathi

      elseif(mrange.EQ.2) then
c ---    X/Y (km)
c ---    Check Easting order
         if(xhi.LT.xlo) then
            x=xhi
            xhi=xlo
            xlo=x
         endif
c ---    Check Northing order
         if(yhi.LT.ylo) then
            y=yhi
            yhi=ylo
            ylo=y
         endif
c ---    Assign to generics
         xe1=xlo
         xe2=xhi
         yn1=ylo
         yn2=yhi

      else
         write(ilst,*)
         write(ilst,*)'ERROR:  invalid range selection'
         write(ilst,*)'Expected 1:Lat/Lon or 2:X/Y'
         write(ilst,*)'Found:  ',mrange
         stop 'Halted -- see list file'
      endif

c --- Process header
      call RDWRHD(itype,mrange,yn1,yn2,xe1,xe2,
     &            nsta,nmap,map,halt132)

c --- Problem?
      if(halt132.NE.'OK') then
         goto 999
      endif
      if(nmap.EQ.0) then
         halt132='No stations found in subdomain'
         goto 999
      endif


c --- Process data records
c ------------------------

10    call RDWRDAT(nsta,nmap,map,itype,halt132,ieof)

c --- Problem?
      if(halt132.NE.'OK') then
         goto 999
      endif

c --- Process next period
      if(ieof.EQ.0) goto 10

999   close(ilst)
      close(iinp)
      close(iout)

      return
      end

c----------------------------------------------------------------------
      subroutine passmap(iutmzni,feasti,fnorthi,pmapi,datumi,utmhemi,
     &                   clat0i,clon0i,clat1i,clat2i,
     &                   halt132)
c----------------------------------------------------------------------
c
c --- SUBDOMN    Version: 1.2        Level: 051128              PASSMAP
c ---            D. Strimaitis, Earth Tech, Inc.
c
c --- Pass the output mapping information to /CONTROL/
c
c --- INPUTS:
c --- OUTPUT:
c           HALT132 - char*132   - Error message
c
c --- PASSMAP called by:  SUBDOMN
c --- PASSMAP calls:      none
c----------------------------------------------------------------------
      include 'control.sdm'

      character*132 halt132
      character*4 utmhemi
      character*8 datumi, pmapi
      character*16 clat1i,clat2i,clat0i,clon0i

      halt132='OK'

c --- Move mapping info into /CONTROL/
      pmap=pmapi
      datum=datumi
      iutmzn=iutmzni
      utmhem=utmhemi
      feast=feasti
      fnorth=fnorthi
      clat0=clat0i
      clon0=clon0i
      clat1=clat1i
      clat2=clat2i
c --- Set logicals for map projection PMAP
      lgeo =.FALSE.
      lutm =.FALSE.
      llcc =.FALSE.
      lps  =.FALSE.
      lem  =.FALSE.
      llaza=.FALSE.
      lttm =.FALSE.
      if(pmap.EQ.'LL') then
         lgeo =.TRUE.
      elseif(pmap.EQ.'UTM') then
         lutm =.TRUE.
      elseif(pmap.EQ.'LCC') then
         llcc =.TRUE.
      elseif(pmap.EQ.'PS') then
         lps  =.TRUE.
      elseif(pmap.EQ.'EM') then
         lem  =.TRUE.
      elseif(pmap.EQ.'LAZA') then
         llaza=.TRUE.
      elseif(pmap.EQ.'TTM') then
         lttm =.TRUE.
      else
         halt132='Unrecognized projection: '//pmap
      endif
c --- Set NIMA date
      datenima='02-21-2003  '

      return
      end

c----------------------------------------------------------------------
      subroutine rdwrhd(itype,mrange,yn1,yn2,xe1,xe2,
     &                  nsta,nmap,map,halt132)
c----------------------------------------------------------------------
c
c --- SUBDOMN    Version: 1.2        Level: 060210               RDWRHD
c ---            D. Strimaitis, Earth Tech, Inc.
c
c --- Process the header records from the SURF.DAT, PRECIP.DAT, or
c     OZONE.DAT file
c
c --- INPUTS:
c             ITYPE - integer    - File type
c                                  1 = SURF.DAT
c                                  2 = PRECIP.DAT
c                                  3 = OZONE.DAT
c            MRANGE - integer    - Coordinate type for range
c                                  1 = LAT/LON
c                                  2 = X,Y
c               YN1 - real       - Lower N. Latitude of range (deg)
c                                  or minimum y (km)
c               YN2 - real       - Upper N. Latitude of range (deg)
c                                  or maximum y (km)
c               XE1 - real       - Lower E. Longitude of range (deg)
c                                  or minimum x (km)
c               XE2 - real       - Upper E. Longitude of range (deg)
c                                  or maximum x (km)
c --- OUTPUT:
c              NSTA - integer    - Number of stations in input file
c              NMAP - integer    - Number of stations for output
c        MAP(maxss) - int. array - List of input stations for output
c           HALT132 - char*132   - Error message
c
c --- UPDATE
c --- V1.0 (051128) - V1.1 (060210)  (DGS)
c                   - Add generic range coordinates
c
c --- RDWRHD called by:  SUBDOMN
c --- RDWRHD calls:      NIMADATE, XTRACTLL, GLOBE1, GLOBE, INCR,
c                        VERDOC
c----------------------------------------------------------------------
      include 'params.sdm'
      include 'control.sdm'
c
      real anem(maxss)
      integer idsta(maxss)
      character*4 cname(maxss)
      character*16 c16name(maxss)
      character*16 clat(maxss),clon(maxss), blank16
      character*64 char64

      integer map(maxss)
      real xnlat(maxss),xelon(maxss)
      real xsta(maxss),ysta(maxss)

      character*4 xyunit,xyuniti,c4hem
      character*8 datumi, pmapi
      character*12 daten,dateni
      character*8 cmapi, cmapo
      character*12 caction
      real*8 vecti(9),vecto(9)

      character*8 axtz
      character*16 dataset,dataver,filtyp(3)
      character*64 datamod
      character*96 comment1, blank96
      character*50 verdoc

      character*132 halt132

      data blank16/'                '/

c --- Set expected dataset name for each of the file types
      data filtyp/'STANDARD_S.DAT  ','STANDARD_P.DAT  ',
     &            'STANDARD_O.DAT  '/

c --- X,Y output in KM
      data xyunit/'  KM'/

      blank96=blank16//blank16//blank16//blank16//blank16//blank16

c --- Set halt string
      halt132='OK'

c --- Obtain COORDS version information
      call COORDSVER(ilst,verdoc)

c --- Set first comment record
      comment1=blank96
      comment1='Produced by SUBDOMN Version: '//ver//'Level: '//level

c --- Establish FIRST header record for output data file
      if(itype.EQ.1) then
         dataver='2.0             '
         dataset='SURF.DAT        '
         datamod='Header structure with NO coordinates'
      elseif(itype.EQ.2) then
         dataver='2.0             '
         dataset='PRECIP.DAT      '
         datamod='Header structure with NO coordinates'
      elseif(itype.EQ.3) then
         dataver='2.1             '
         dataset='OZONE.DAT       '
         datamod='Header structure with coordinates'
      endif
      write(iout,'(2a16,a64)') dataset,dataver,datamod

c --- Write similar record for list file of coordinates
      if(itype.EQ.1) then
         dataver='1.0             '
         dataset='SURF_STN.LST    '
         datamod='Coordinates of SURFACE MET STATIONS Extracted'
      elseif(itype.EQ.2) then
         dataver='1.0             '
         dataset='PRECIP_STN.LST  '
         datamod='Coordinates of PRECIPITATION STATIONS Extracted'
      elseif(itype.EQ.3) then
         dataver='1.0             '
         dataset='OZONE_STN.LST   '
         datamod='Coordinates of OZONE STATIONS Extracted'
      endif
      write(ilst,'(2a16,a64)') dataset,dataver,datamod


c --- Assume formatted input file
      read(iinp,'(2a16,a64)') dataset,dataver,datamod
      read(iinp,*) ncomment
      if(dataset.NE.filtyp(itype)) then
         halt132='RDWRHD: Invalid data file: '//dataset
         return
      endif

c --- Documentation (data and list files)
      ncomm=6+ncomment
      nclst=ncomm+1
      if(itype.EQ.3) ncomm=nclst
      write(iout,*) ncomm
      write(iout,'(a96)') comment1
      write(iout,'(a48)')
     &     'Stations Extracted from MMS Master Data File ---'
      write(iout,'(2a16,a64)') dataset,dataver,datamod

      write(ilst,*) nclst
      write(ilst,'(a96)') comment1
      write(ilst,'(a48)')
     &     'Stations Extracted from MMS Master Data File ---'
      write(ilst,'(2a16,a64)') dataset,dataver,datamod

      do i=1,ncomment
         comment1=blank96
         read(iinp,'(a96)') comment1
         write(iout,'(a96)') comment1
         write(ilst,'(a96)') comment1
      enddo

      if(mrange.EQ.1) then
         xlatlo=yn1
         xlathi=yn2
         xlonlo=xe1
         xlonhi=xe2
         write(iout,'(a22)')'Extraction Range ---  '
         write(iout,*)'  N. Latitude:  ',xlatlo,xlathi
         write(iout,*)'  E. Longitude: ',xlonlo,xlonhi

         write(ilst,'(a22)')'Extraction Range ---  '
         write(ilst,*)'  N. Latitude:  ',xlatlo,xlathi
         write(ilst,*)'  E. Longitude: ',xlonlo,xlonhi

      elseif(mrange.EQ.2) then
         write(iout,'(a22)')'Extraction Range ---  '
         write(iout,*)'  X(km) Easting : ',xe1,xe2
         write(iout,*)'  Y(km) Northing: ',yn1,yn2

         write(ilst,'(a22)')'Extraction Range ---  '
         write(ilst,*)'  X(km) Easting : ',xe1,xe2
         write(ilst,*)'  Y(km) Northing: ',yn1,yn2
      endif

c --- Report COORDS version
      comment1=blank96
      comment1(1:36)='Internal Coordinate Transformations '
      comment1(37:86)=verdoc(1:50)
      write(ilst,'(a96)') comment1
      if(itype.EQ.3)write(iout,'(a96)') comment1

c --- Read input projection information (LL here)
      read(iinp,'(a8)') pmapi
      read(iinp,'(a8,a12)') datumi,dateni
      read(iinp,'(a4)') xyuniti
c --- Read time range, time zone, and number of stations
      read(iinp,*)ibyr,ibjul,ibhr,ieyr,iejul,iehr,ibtz,nsta
      xbtz=FLOAT(ibtz)+.01
      call BASRUTC(xbtz,axtz)
      ibsec=0
      iesec=0
      if(itype.eq.3) then
c ---    Set begin time at the START of the first hour in OZONE.DAT
         nhr=-1
         call INCR(ilst,ibyr,ibjul,ibhr,nhr)
      endif

c --- Write projection-related header information to data file
      if(itype.EQ.3) then
c ---    OZONE file needs coordinates in user's system
         write(iout,'(a8)') pmap
         if(LUTM) then
            write(iout,'(i4,a4)') iutmzn,utmhem
         elseif(LLCC) then
            write(iout,'(4a16)') clat0,clon0,clat1,clat2
         elseif(LPS) then
            write(iout,'(3a16)') clat0,clon0,clat1
         elseif(LEM.or.LLAZA.or.LTTM) then
            write(iout,'(2a16)') clat0,clon0
         endif
         if(LLCC.or.LLAZA.or.LTTM) then
            write(iout,*) feast,fnorth
         endif
         call NIMADATE(daten)
         write(iout,'(a8,a12)') datum,daten
         write(iout,'(a4)') xyunit
         write(iout,'(a8)') axtz
      else
c ---    SURF and PRECIP files have NO coordinates
         write(iout,'(a8)') 'NONE    '
      endif

c --- Write projection-related header information to list file
      write(ilst,'(a8)') pmap
      if(LUTM) then
         write(ilst,'(i4,a4)') iutmzn,utmhem
      elseif(LLCC) then
         write(ilst,'(4a16)') clat0,clon0,clat1,clat2
      elseif(LPS) then
         write(ilst,'(3a16)') clat0,clon0,clat1
      elseif(LEM.or.LLAZA.or.LTTM) then
         write(ilst,'(2a16)') clat0,clon0
      endif
      if(LLCC.or.LLAZA.or.LTTM) then
         write(ilst,*) feast,fnorth
      endif
      call NIMADATE(daten)
      write(ilst,'(a8,a12)') datum,daten
      write(ilst,'(a4)') xyunit

c --- Store station list
      if(itype.EQ.1) then
         do n=1,nsta
            read(iinp,*)idsta(n),cname(n),clat(n),clon(n),anem(n)
         enddo
      elseif(itype.EQ.2) then
         do n=1,nsta
            read(iinp,*)idsta(n),cname(n),clat(n),clon(n)
         enddo
      elseif(itype.EQ.3) then
         do n=1,nsta
            read(iinp,*)c16name(n),clat(n),clon(n)
         enddo
      endif

c --- Set up for coordinate transformations
c -------------------------------------------------
c --- Scale factor for Tangential TM projection
      tmsone=1.00000
c -----------------
c --- Define Output 
c -----------------
c --- Set utm zone for coordinate calls
      iutmo=iutmzn
      if(utmhem.EQ.'S   ' .AND. iutmzn.LT.900) iutmo=-iutmo
c --- Set mapping code
      cmapo=pmap
      if(cmapo.EQ.'TTM     ') cmapo='TM      '
c --- Extract lat/lon from character fields
      if(clat0.NE.blank16) call XTRACTLL(ilst,'LAT ',clat0,rlat0)
      if(clon0.NE.blank16) call XTRACTLL(ilst,'LON ',clon0,rlon0)
      if(clat1.NE.blank16) call XTRACTLL(ilst,'LAT ',clat1,rlat1)
      if(clat2.NE.blank16) call XTRACTLL(ilst,'LAT ',clat2,rlat2)
c --- Adjust projection information if needed
      if(LEM) then
c ---    Equatorial Mercator projection matches at 0.0N, 
c ---    and places the northing origin at 0.0N
         rlat0=0.0
         rlat1=0.0
         rlat2=0.0
      endif
c -----------------
c --- Define Input 
c -----------------
c --- Set mapping codes
      cmapi='LL      '
      dum0=0.0
      idum0=0
c -------------------------
c --- Set conversion vector
c -------------------------
      call GLOBE1(cmapi,idum0,tmsone,dum0,dum0,dum0,dum0,
     &            dum0,dum0,
     &            cmapo,iutmo,tmsone,rlat1,rlat2,rlat0,rlon0,
     &            feast,fnorth,
     &            caction,vecti,vecto)


c --- Convert station lat/lon to output x/y
      do is=1,nsta
         call XTRACTLL(ilst,'LAT ',clat(is),xnlat(is))
         call XTRACTLL(ilst,'LON ',clon(is),xelon(is))
         call GLOBE(ilst,caction,datumi,vecti,datum,vecto,
     &        xelon(is),xnlat(is),xsta(is),ysta(is),izone,c4hem)
      enddo

c --- Process location of stations relative to requested range
      nmap=0
      if(mrange.EQ.1) then
         do is=1,nsta
c ---       Range test will not work across 0/360 cut!
            if(xnlat(is).GE.xlatlo .AND. xnlat(is).LE.xlathi) then
               if(xelon(is).GE.xlonlo .AND. xelon(is).LE.xlonhi) then
                  nmap=nmap+1
                  map(nmap)=is
               endif
            endif
         enddo
      elseif(mrange.EQ.2) then
         do is=1,nsta
            if(xsta(is).GE.xe1 .AND. xsta(is).LE.xe2) then
               if(ysta(is).GE.yn1 .AND. ysta(is).LE.yn2) then
                  nmap=nmap+1
                  map(nmap)=is
               endif
            endif
         enddo
      endif

c --- Exit now if no stations are in selected region
      if(nmap.EQ.0) return


c --- Write new station list
      if(itype.EQ.1) then
         write(ilst,*)
         write(ilst,*)'Number of SURFACE stations (Group 7): ',nmap
         write(ilst,*)
         write(ilst,*)
         write(iout,*)ibyr,ibjul,ibhr,ieyr,iejul,iehr,ibtz,nmap
         do n=1,nmap
            j=map(n)
            write(iout,*)idsta(j)
            if(n.LT.10) then
               write(ilst,101)n,cname(j),idsta(j),xsta(j),ysta(j),
     &                        xbtz,anem(j)
            elseif(n.LT.100) then
               write(ilst,102)n,cname(j),idsta(j),xsta(j),ysta(j),
     &                        xbtz,anem(j)
            else
               write(ilst,103)n,cname(j),idsta(j),xsta(j),ysta(j),
     &                        xbtz,anem(j)
            endif
         enddo
      elseif(itype.EQ.2) then
         write(ilst,*)
         write(ilst,*)'Number of PRECIP stations (Group 9): ',nmap
         write(ilst,*)
         write(ilst,*)
         write(iout,*)ibyr,ibjul,ibhr,ieyr,iejul,iehr,ibtz,nmap
         do n=1,nmap
            j=map(n)
            write(iout,*) idsta(j)
            if(n.LT.10) then
               write(ilst,201) n,cname(j),idsta(j),xsta(j),ysta(j)
            elseif(n.LT.100) then
               write(ilst,202) n,cname(j),idsta(j),xsta(j),ysta(j)
            else
               write(ilst,203) n,cname(j),idsta(j),xsta(j),ysta(j)
            endif
         enddo
      elseif(itype.EQ.3) then
         write(ilst,*)
         write(ilst,*)'Number of OZONE stations: ',nmap
         write(ilst,*)
         write(ilst,*)
         write(iout,*)ibyr,ibjul,ibhr,ibsec,ieyr,iejul,iehr,iesec
         write(iout,*)nmap
         do n=1,nmap
            j=map(n)
            write(iout,301) c16name(j),xsta(j),ysta(j)
            write(ilst,301) c16name(j),xsta(j),ysta(j)
         enddo
      endif

c --- CALMET control file record formats
101   format('! SS',i1,'  =','''',a4,'''',i9,2f13.3,f5.0,f7.1,'  !')
102   format('! SS',i2,' =','''',a4,'''',i9,2f13.3,f5.0,f7.1,'  !')
103   format('! SS',i3,'=','''',a4,'''',i9,2f13.3,f5.0,f7.1,'  !')
201   format('! PS',i1,'  =','''',a4,'''',i9,2f13.3,'  !')
202   format('! PS',i2,' =','''',a4,'''',i9,2f13.3,'  !')
203   format('! PS',i3,'=','''',a4,'''',i9,2f13.3,'  !')
301   format('''',a16,'''',2f13.3)

      return
      end

c----------------------------------------------------------------------
      subroutine rdwrdat(nsta,nmap,map,itype,halt132,ieof)
c----------------------------------------------------------------------
c
c --- SUBDOMN  Version: 1.2       Level: 010419                RDWRDAT
c ---          D. Strimaitis, Earth Tech
c
c --- Read record for one time period and write output record for
c --- subset of stations (prepared for formatted version of SURF.DAT,
c --- PRECIP.DAT, or OZZONE.DAT files).
c
c --- INPUTS:
c              NSTA - integer    - Number of stations in input file
c              NMAP - integer    - Number of stations for output
c         MAP(NMAP) - int. array - List of input stations for output
c             ITYPE - integer    - File type
c                                  1 = SURF.DAT
c                                  2 = PRECIP.DAT
c                                  3 = OZONE.DAT
c
c --- OUTPUT:
c           HALT132 - char*132   - Error message
c              IEOF - integer    - End-of-File flag (0,1)
c
c --- RDWRDAT called by:  SUBDOMN
c --- RDWRDAT calls:      INCR
c------------------------------------------------------------------------------
      include 'params.sdm'
c
      character*132 halt132

      integer map(maxss)

      integer iceil1(maxss),icc1(maxss),irh1(maxss),ipcode1(maxss)
      integer iceil2(maxss),icc2(maxss),irh2(maxss),ipcode2(maxss)

      real ws1(maxss),wd1(maxss),tempk1(maxss),pres1(maxss)
      real ws2(maxss),wd2(maxss),tempk2(maxss),pres2(maxss)
      real prec1(maxss), ozone1(maxss)
      real prec2(maxss), ozone2(maxss)

      ieof=0
      halt132='OK'

      ibsec=0
      iesec=0

      if(nsta.GT.maxss) then
         halt132='RDWRDAT:  Number of stations exceeds array limit'
         return
      endif

      if(itype.EQ.1) then
         read(iinp,*,end=999)iyr,ijul,ihr,
     &                      (ws1(n),wd1(n),iceil1(n),icc1(n),tempk1(n),
     &                       irh1(n),pres1(n),ipcode1(n),n=1,nsta)
c ---    Assign to output stations
         do i=1,nmap
            j=map(i)
            ws2(i)=ws1(j)
            wd2(i)=wd1(j)
            iceil2(i)=iceil1(j)
            icc2(i)=icc1(j)
            tempk2(i)=tempk1(j)
            irh2(i)=irh1(j)
            pres2(i)=pres1(j)
            ipcode2(i)=ipcode1(j)
         enddo
c ---    Write output record
         write(iout,'(3i4)')iyr,ijul,ihr
         write(iout,1) (ws2(n),wd2(n),iceil2(n),icc2(n),tempk2(n),
     &                  irh2(n),pres2(n),ipcode2(n),n=1,nmap)
1        format(1x,f8.3,1x,f8.3,1x,i4,1x,i4,1x,f8.3,1x,i4,1x,f8.3,
     &          1x,i4)

      elseif(itype.EQ.2) then
         read(iinp,*,end=999)iyr,ijul,ihr,(prec1(n),n=1,nsta)
c ---    Assign to output stations
         do i=1,nmap
            j=map(i)
            prec2(i)=prec1(j)
         enddo
c ---    Write output record
         write(iout,2)iyr,ijul,ihr,(prec2(n),n=1,nmap)
2        format(3i4,1x,10f9.3,50(/,13x,10f9.3))

      elseif(itype.EQ.3) then
         read(iinp,*,end=999)iyr,ijul,ihr,(ozone1(n),n=1,nsta)
c ---    Assign to output stations
         do i=1,nmap
            j=map(i)
            ozone2(i)=ozone1(j)
         enddo
c ---    Need begin time in OZONE.DAT
         ibyr=iyr
         ibjul=ijul
         ibhr=ihr
         nhr=-1
         call INCR(ilst,ibyr,ibjul,ibhr,nhr)
c ---    Write output record
         write(iout,3)ibyr,ibjul,ibhr,ibsec,iyr,ijul,ihr,iesec,
     &                (ozone2(n),n=1,nmap)
3        format(2(2i4,i3.2,i5.4,1x),10f9.2,50(/,34x,10f9.2))

      else
         halt132='Halted in RDWRDAT: itype not expected value' 
      endif

      return

999   ieof=1
      return
      end



c ---------------------------------------------------------------------
c --- Include the module routines here
      include 'coordlib.for'
c ---------------------------------------------------------------------


c----------------------------------------------------------------------
      subroutine xtractll(io,type,clatlon,rlatlon)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 2.2      Level: 030402               XTRACTLL
c                D. Strimaitis   EarthTech
c
c --- PURPOSE:  Extract the real latitude or longitude from a character
c               string that contains the N/S or E/W convention
c               character, and express result as either North Latitude
c               or East Longitude
c
c --- UPDATE
c --- V2.1 (030402) from V2.0 (010713) (DGS)
c               - Add TYPE argument for QA
c               - Change format (f16) to (f16.0) to satisfy different
c                 compilers
c
c
c --- INPUTS:
c               IO - integer    - Unit number for list file output
c             TYPE - char*4     - LAT or LON
c          CLATLON - char*16    - Latitude or longitude (degrees), with
c                                 1 character that denotes convention
c                                 (e.g. 'N  45.222' or  '-35.999s')
c
c --- OUTPUT:
c          RLATLON - real       - North Latitude or East Longitude
c                                 (degrees)
c
c --- XTRACTLL called by: (utility)
c --- XTRACTLL calls:     DEBLNK, ALLCAP
c----------------------------------------------------------------------
c
c --- Include parameter statements
      include 'params.cal'

      character*1 cstor1(mxcol),cstor2(mxcol)
      character*16 clatlon, clatlon2
      character*4 type
      logical ltype

      ltype=.FALSE.

c --- Initialize character variables for output
      clatlon2='                '
      do i=1,20
         cstor2(i)=' '
      enddo

c --- Was valid type provided?
      if(type.NE.'LAT ' .AND. type.NE.'LON ') then
         write(io,*) 'XTRACTLL:  FATAL ERROR reported when ',
     &               'extracting Latitude/Longitude'
         write(io,*) 'Invalid type:  ',type
         write(io,*) 'Expected LAT or LON'
         write(*,*)
         stop 'Halted in XTRACTLL -- see list file'
      endif

c --- Pass c*16 string into storage array 1
      do i=1,16
         cstor1(i)=clatlon(i:i)
      enddo
c --- Pad out to position 20
      do i=17,20
         cstor1(i)=' '
      enddo

c --- Remove blank characters from string, place in storage array 2
c --- (Use a 20-character field here for a margin at end of string)
      call DEBLNK(cstor1,1,20,cstor2,nlim)
c
c --- Convert lower case letters to upper case
      call ALLCAP(cstor2,nlim)

c --- Interpret valid convention character (N,S,E,W)
      nchar=0
      ichar=0
      ilat=0
      ilon=0
   
      do i=1,nlim
         if(cstor2(i).EQ.'N') then
            ilat=1
            ichar=i
            nchar=nchar+1
         elseif(cstor2(i).EQ.'S') then
            ilat=2
            ichar=i
            nchar=nchar+1
         elseif(cstor2(i).EQ.'W') then
            ilon=1
            ichar=i
            nchar=nchar+1
         elseif(cstor2(i).EQ.'E') then
            ilon=2
            ichar=i
            nchar=nchar+1
         endif
      enddo

c --- Was 1 valid character found?
      if(nchar.NE.1) then
         write(io,*) 'XTRACTLL:  FATAL ERROR reported when ',
     &               'extracting Latitude/Longitude'
         write(io,*) 'N,S,E,W character is missing or repeated'
         write(io,*) 'Lat/Lon = ',clatlon
         write(*,*)
         stop 'Halted in XTRACTLL -- see list file'
      endif

c --- Was valid character the right type?
      if(type.EQ.'LAT ' .AND. ilat.EQ.0) ltype=.TRUE.
      if(type.EQ.'LON ' .AND. ilon.EQ.0) ltype=.TRUE.
      if(LTYPE) then
         write(io,*) 'XTRACTLL:  FATAL ERROR reported when ',
     &               'extracting Latitude/Longitude'
         write(io,*) 'N,S,E,W character does not match type'
         write(io,*) 'Lat/Lon = ',clatlon
         write(io,*) 'type    = ',type
         write(*,*)
         stop 'Halted in XTRACTLL -- see list file'
      endif

c --- Remove character from string
      do i=ichar,nlim
         cstor2(i)=cstor2(i+1)
      enddo

c --- Search for position of decimal point
      ipt=0
      do i=1,nlim
         if(cstor2(i).EQ.'.') ipt=i
      enddo

c --- Add a decimal point if needed
      if(ipt.EQ.0) then
         cstor2(nlim)='.'
      endif

c --- Pass resulting "number" back into c*16 variable
      do i=1,nlim
         clatlon2(i:i)=cstor2(i)
      enddo

c --- Get real part
      read(clatlon2,'(f16.0)') rlatlon

c --- Convert to either N. Lat. or E. Lon., if needed
      if(ilat.EQ.2) then
         rlatlon=-rlatlon
      elseif(ilon.EQ.1) then
         rlatlon=-rlatlon
      endif

c --- Condition longitude to be -180 to +180
      if(ilon.GT.0) then
         if(rlatlon.GT.180.) then
            rlatlon=rlatlon-360.
         elseif(rlatlon.LT.-180.) then
            rlatlon=rlatlon+360.
         endif
      endif

      return
      end
c----------------------------------------------------------------------
      subroutine incr(io,iyr,ijul,ihr,nhrinc)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 2.2      Level: 000602                   INCR
c                J. Scire, SRC
c
c --- PURPOSE:  Increment the time and date by "NHRINC" hours
c
c --- UPDATE
c ---               000602  (DGS): add message to "stop"
c ---               980304  (DGS): Allow for a negative "increment" of
c                                  up to 24 hours
c ---               980304  (DGS): Allow for arbitrarily large nhrinc
c
c --- INPUTS:
c       IO     - integer - Unit number for list file output
c       IYR    - integer - Current year
c       IJUL   - integer - Current Julian day
c       IHR    - integer - Current hour (00-23)
c       NHRINC - integer - Time increment (hours)
c
c               NOTE: "NHRINC" must >= -24
c                      Hour is between 00-23
c
c --- OUTPUT:
c       IYR    - integer - Updated year
c       IJUL   - integer - Updated Julian day
c       IHR    - integer - Updated hour (00-23)
c
c --- INCR called by: host subroutines
c --- INCR calls:     none
c----------------------------------------------------------------------
c
c --- Check nhrinc
      if(nhrinc.lt.-24) then
         write(io,*)'ERROR IN SUBR. INCR -- Invalid value of NHRINC ',
     1   '-- NHRINC = ',nhrinc
         write(*,*)
         stop 'Halted in INCR -- see list file.'
      endif

c --- Save increment remaining (needed if nhrinc > 8760)
      nleft=nhrinc
c
c --- Process change in hour
      if(nhrinc.gt.0)then
c
10       ninc=MIN0(nleft,8760)
         nleft=nleft-ninc
c
c ---    Increment time
         ihr=ihr+ninc
         if(ihr.le.23)return
c
c ---    Increment day
         ijul=ijul+ihr/24
         ihr=mod(ihr,24)
c
c ---    ILEAP = 0 (non-leap year) or 1 (leap year)
         if(mod(iyr,4).eq.0)then
            ileap=1
         else
            ileap=0
         endif
c
         if(ijul.gt.365+ileap) then
c ---       Update year
            iyr=iyr+1
            ijul=ijul-(365+ileap)
         endif
c
c ---    Repeat if more hours need to be added
         if(nleft.GT.0) goto 10
c
      elseif(nhrinc.lt.0)then
c ---    Decrement time
         ihr=ihr+nhrinc
         if(ihr.lt.0)then
            ihr=ihr+24
            ijul=ijul-1
            if(ijul.lt.1)then
               iyr=iyr-1
               if(mod(iyr,4).eq.0)then
                  ijul=366
               else
                  ijul=365
               endif
            endif
         endif
      endif
c
      return
      end
c----------------------------------------------------------------------
      subroutine deblnk(cstor1,ilim1,il2,cstor2,nlim)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 2.2      Level: 030402                 DEBLNK
c ---            J. Scire, Earth Tech, Inc.
c
c --- PURPOSE:  Remove all blank or "+" characters from the character 
c               string within delimiters
c
c --- UPDATE
c --- V2.1 (030402) from V2.0 (980918) (DGS)
c               - Split DEBLNK action (removes ' ', '+') into 
c                 DEBLNK and DEPLUS(new)
c
c --- INPUTS:
c
c    CSTOR1(mxcol) - character*1 array - Input character string
c            ILIM1 - integer           - Array element at which search
c                                        for blanks begins
c              IL2 - integer           - Array element at which search
c                                        for blanks ends
c        Parameters: MXCOL
c
c --- OUTPUT:
c
c    CSTOR2(mxcol) - character*1 array - Output character string
c                                        (without blanks within text)
c             NLIM - integer           - Length of output string
c                                        (characters)
c
c --- DEBLNK called by:  (utility)
c --- DEBLNK calls:      none
c----------------------------------------------------------------------
c
c --- Include parameter statements
      include 'params.cal'
c
      character*1 cstor1(mxcol),cstor2(mxcol),cblnk
      data cblnk/' '/
c
      ind=0
      do 10 i=ilim1,il2
      if(cstor1(i).eq.cblnk)go to 10
c
c --- transfer non-blank character into output array
      ind=ind+1
      cstor2(ind)=cstor1(i)
10    continue
      nlim=ind
      if(ind.eq.mxcol)return
c
c --- pad rest of output array
      indp1=ind+1
      do 20 i=indp1,mxcol
      cstor2(i)=cblnk
20    continue
      return
      end
c----------------------------------------------------------------------
      subroutine allcap(cstor2,nlim)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 2.2      Level: 950122                 ALLCAP
c ---            J. Scire, SRC
c
c --- PURPOSE:  Convert all lower case letters within a character
c               string to upper case
c
c --- INPUTS:
c
c    CSTOR2(mxcol) - character*1 array - Input character string
c             NLIM - integer           - Length of string (characters)
c        Parameters: MXCOL
c
c --- OUTPUT:
c
c    CSTOR2(mxcol) - character*1 array - Output character string with
c                                        lower case letters converted
c                                        to upper case
c
c --- ALLCAP called by:  READIN
c --- ALLCAP calls:      none
c----------------------------------------------------------------------
c
c --- Include parameter statements
      include 'params.cal'
c
      character*1 cstor2(mxcol),cchar,clc(29),cuc(29)
c
      data clc/'i','n','x','a','e','o','u','b','c','d','f','g','h',
     1 'j','k','l','m','p','q','r','s','t','v','w','y','z','-','.',
     2 '*'/
      data cuc/'I','N','X','A','E','O','U','B','C','D','F','G','H',
     1 'J','K','L','M','P','Q','R','S','T','V','W','Y','Z','-','.',
     2 '*'/
c
      do 100 i=1,nlim
      cchar=cstor2(i)
c
      do 50 j=1,29
      if(cchar.eq.clc(j))then
         cstor2(i)=cuc(j)
         go to 52
      endif
50    continue
52    continue
100   continue
c
      return
      end

c----------------------------------------------------------------------
      subroutine basrutc(xbtz,axtz)
c----------------------------------------------------------------------
c
c --- CALUTILS   Version: 2.5      Level: 041123                BASRUTC
c ---            F.Robe, Earth Tech
c
c --- PURPOSE:  Converts real base time zone  to character string
c               UTC time zone
c
c --- INPUT:
c          XBTZ - real      - base time zone (old convention: positive
c                             in North America i.e. opposite to UTC)

c --- OUTPUT:
c          AXTZ - real      - time zone (international convention: 
c                             relative to UTC/GMT)UTC-HHMM
c
c --- BASRUTC called by:  host subroutines
c --- BASRUTC calls:      none
c----------------------------------------------------------------------
      character*8 axtz

      ixbtz=int(xbtz) 
c     convert fractional real to minutes
      imin=(xbtz-ixbtz)*60
      ixbtz=ixbtz*100+imin

c --- Define time as "UTC-HHMM" (hours/minutes)
      axtz(1:3)="UTC"

c --- Flip sign as base time zone is minus UTC zone
      if (xbtz.gt.0.) then
         axtz(4:4)="-"
      else
         axtz(4:4)="+"
      endif
c --- Make sure time zone is written as 4 digits
      if (abs(xbtz).lt.10.) then
         axtz(5:5)="0"
         write(axtz(6:8),'(i3)')abs(ixbtz)
      else
         write(axtz(5:8),'(i4)')abs(ixbtz)
      endif

      return
      end
